import inspect
import os

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import JsonResponse

from .openai import OpenAIClient


COMPLETION_PROMPT = inspect.cleandoc(
    """You are assisting a user in writing content for their website.
    The user has provided some initial text (following the colon).
    Assist the user in writing the remaining content:"""
)

CORRECTION_PROMPT = inspect.cleandoc(
    """You are assisting a user in writing content for their website.
    The user has provided some text (following the colon).
    Return the provided text but with corrected grammar, spelling and punctuation.
    Do not add additional punctuation, quotation marks or change any words:"""
)


def build_openai_client() -> OpenAIClient:
    try:
        api_key = settings.OPENAI_API_KEY
        return OpenAIClient(api_key=api_key)
    except AttributeError:
        raise ImproperlyConfigured(
            "The OPENAI_API_KEY setting must be configured to use Wagtail AI"
        )


def ai_process(request):
    text = request.GET.get("text")
    action = request.GET.get("action", "completion")
    prompt = COMPLETION_PROMPT if action == "completion" else CORRECTION_PROMPT
    full_prompt = "\n".join([prompt, text])
    client = build_openai_client()
    message = client.chat(full_prompt)
    # Remove extra blank lines returned by the API
    message = os.linesep.join([s for s in message.splitlines() if s])
    return JsonResponse({"message": message})



import wagtail.admin.rich_text.editors.draftail.features as draftail_features

from django.urls import include, path
from django.views.i18n import JavaScriptCatalog
from wagtail import hooks

from .views import ai_process


@hooks.register("register_admin_urls")
def register_admin_urls():
    urls = [
        path(
            "jsi18n/",
            JavaScriptCatalog.as_view(packages=["wagtail_ai"]),
            name="javascript_catalog",
        ),
        path(
            "process/",
            ai_process,
            name="process",
        ),
    ]

    return [
        path(
            "ai/",
            include(
                (urls, "wagtail_ai"),
                namespace="wagtail_ai",
            ),
        )
    ]


class ControlFeature(draftail_features.Feature):
    def construct_options(self, options):
        return None


@hooks.register("register_rich_text_features")
def register_ai_feature(features):
    features.default_features.append("ai")
    features.register_editor_plugin(
        "draftail",
        "ai",
        ControlFeature(
            js=["wagtail_ai/wagtail-ai.js"], css={"all": ["wagtail_ai/main.css"]}
        ),
    )



from django.conf import settings
from django.exceptions import ImproperlyConfigured


try:
    settings.OPENAI_API_KEY
except AttributeError:
    raise ImproperlyConfigured(
        "The OPENAI_API_KEY setting must be configured to use Wagtail AI"
    )
